@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_service - remote control daemon for Nokia N900 phone#
-# $Id: smscon_service.py 163 2012-01-22 22:21:37Z yablacky $
+# $Id: smscon_service.py 168 2012-05-12 19:04:44Z yablacky $
 ##############################################################
 
 # standard modules
@@ -524,8 +524,9 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     Process the received command message.
     """
     # Check if Command is a specific valid SMSCON command:
-    CommandList = [smscnf[cmd] for cmd in smscnf.getSmsCommandList() if smscnf[cmd] != ""]
-    if not Command in CommandList:
+    if not (Command in [smscnf[cmd] for cmd in smscnf.getSmsCommandList() if smscnf[cmd] != '']
+            or smscnf.parseSmsCommand(Command, smscnf.COM_CUSTOM)
+            or smscnf.parseSmsCommand(Command, smscnf.COM_SHELL)):
         if LogIfCmdUnknown:
             LOGGER.error( '"%s" is not a valid %s command.' % (Command, smscon.NAME) )
             LogBatteryChargeInfo()
@@ -543,6 +544,7 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     LOGGER.info('Received %s command [%s] via %s %s. Acknowledge message %s.' % (smscon.NAME, Command, CommandOrigin, ExtraInfo,
                     IF(smscnf.COMMANDREPLY == 'yes', 'enabled', 'disabled')))
     LogBatteryChargeInfo()
+    CommandArgs = {}
 
     # First of all, make phone silent
     EnableDeviceSilence(smscnf.SILENCEDEVICE == 'yes')
@@ -635,17 +637,11 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
         DisableGpsAcquiring()
         SetAutoConnectBehaviour(ConnectionBehaviour)
 
-    elif Command == smscnf.COM_CUSTOM:
-        PerformShellScript()
-
     elif Command == smscnf.COM_CUSTOMLOG:
         PerformShellScriptResult()
 
     elif Command == smscnf.COM_LOG:
-        RevealThat('log', 'The smscon Log file', Attachment = LOGGER.LogFilename, Attachhint = 'text/plain')
-
-    elif Command == smscnf.COM_SHELL:
-        PerformDirectShellCommand(Command)
+        RevealThat('log', 'The %s log file' % smscon.NAME, Attachment = LOGGER.LogFilename, Attachhint = 'text/plain')
 
     elif Command == smscnf.COM_ALARM:
         PerformAlarm()
@@ -653,6 +649,18 @@ def ProcessCommand(Command, SenderPhoneNumber=None, CommandOrigin=None, ExtraInf
     elif Command == smscnf.COM_RESTART:
         RestartDaemon()
 
+    elif smscnf.parseSmsCommand(Command, smscnf.COM_CUSTOM, CommandArgs):
+        os.putenv('SMSCON_COMMANDORIGIN', CommandOrigin)
+        os.putenv('SMSCON_SENDERNUMBER', smscnf.SENDERNUMBER)
+        os.putenv('SMSCON_MASTERNUMBER', smscnf.MASTERNUMBER)
+        PerformShellScript(CommandArgs['args'])
+
+    elif smscnf.parseSmsCommand(Command, smscnf.COM_SHELL):
+        os.putenv('SMSCON_COMMANDORIGIN', CommandOrigin)
+        os.putenv('SMSCON_SENDERNUMBER', smscnf.SENDERNUMBER)
+        os.putenv('SMSCON_MASTERNUMBER', smscnf.MASTERNUMBER)
+        PerformDirectShellCommand(Command)
+
     else:
         LOGGER.debug('Ignoring unknown command: [%s].' % Command)
 
@@ -784,9 +792,10 @@ def EnableDeviceSilence(Enable = True):
 #   run custom scripts
 #############################################
 
-def PerformShellScript():
+def PerformShellScript(args = []):
     """
     Execute user editable script.
+    @param array args Parameters to pass to the script.
     @return void
     """
     Filename = os.path.join(InstPath, smscon.ScriptFile)
@@ -795,15 +804,19 @@ def PerformShellScript():
     if pidof(Filename) == '':
         DeleteFile(Logname)
 
-    if IsFileReadable(Filename):
-        # run script file
-        os.system( '"%s" 1>>"%s" 2>&1&' % (Filename, Logname) )
-        LOGGER.info(         'Script file (%s) executes in BG.' % Filename)
-        RevealThat('script', 'script file (%s) executes in BG.' % Filename)        
-    else:
+    if not IsFileReadable(Filename):
         LOGGER.error(        'script file (%s) not found.' % Filename)
         RevealThat('script', 'script file (%s) not found.' % Filename)        
         DeleteFile(Logname)
+        return;
+    # run script file
+    try:
+        os.system( '"%s" %s 1>>"%s" 2>&1&' % (Filename, ' '.join([shellArg(a) for a in args]), Logname) )
+        LOGGER.info(         'Script %s executes in BG.' % str(args))
+        RevealThat('script', 'Script %s executes in BG.' % str(args))
+    except Exception, e:
+        LOGGER.error('Failed to execute script %s: %s' % (str(args), e))
+        RevealThat('shell', ['Failed to execute script %s.' % str(args)])        
 
 def PerformShellScriptResult():
     """
@@ -828,10 +841,10 @@ def PerformDirectShellCommand(Command):
         Result = os.popen(Command).read()
     except:
         LOGGER.error('Failed to run shell command (%s).' % Command)
-        RevealThat('shell', ['command failed: %s.' % Command])        
+        RevealThat('shell', ['Command failed: %s.' % Command])        
     else:
         LOGGER.info('Shell command (%s) executed.' % Command)
-        RevealThat('shell', ['command: %s\n' % Command, 'result:\n', Result])        
+        RevealThat('shell', ['Command: %s\n' % Command, 'Result:\n', Result])        
 
 #############################################
 #   show SMS commands
@@ -1853,7 +1866,8 @@ def DoCommandLine(args):
                 PerformPhonecall(smscnf.MASTERNUMBER)
 
             elif Func == 'script': # test mode for script
-                PerformShellScript()
+                PerformShellScript(args)
+                args = [];
 
             else:
                 LOGGER.error('Test option error (%s).' % Func)
@@ -1865,7 +1879,7 @@ def DoCommandLine(args):
 
             LOGGER.warning('Simulating received SMS message from MASTERNUMBER with "%s" command.' % Command)
             ProcessCommand(Command, SenderPhoneNumber = smscnf.MASTERNUMBER,
-                           ExtraInfo = 'SIMULATION!', LogIfCmdUnknown = True)    
+                           ExtraInfo = 'SIMULATION!', CommandOrigin = 'CMDLINE', LogIfCmdUnknown = True)    
         else:
             LOGGER.warning('Option error (%s) - ignored' % Mode)
             return CommandLineStatus
