@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 165 2012-04-14 07:10:08Z yablacky $
+# $Id: smsconlib.py 168 2012-05-12 19:04:44Z yablacky $
 ##############################################################
 
 # standard modules
@@ -34,7 +34,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.17/src/opt/smscon/smsconlib.py 165 2012-04-14 07:10:08Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.9.18/src/opt/smscon/smsconlib.py 168 2012-05-12 19:04:44Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -862,6 +862,45 @@ class UserConfiguration(Configuration):
         """@return list of config vars that are SMS commands"""
         return [cmd for cmd in self._ConfigVars if cmd in self and cmd.startswith(self.SmsCommandPreFix)]
 
+    def parseSmsCommand(self, CmdText, CmdPattern, ParametersFound = {}):
+        """
+        Check if a text matches a given command pattern and extract command parameters.
+        Command parameters are defined by * in the pattern. Many * may occur and each defines
+        a separate parameter unless they are consecutive. Two or more consecutive * define
+        a single parameter which declares itself and all the parameters up to the last one
+        on the right hand side to be optional.
+        If optional parameters are omitted, the non-* text between them must be omitted as well.
+        @param string CmdText: command text being checked.
+        @param string CmdPattern: pattern to check against.
+        @param dict ParametersFound: ['args'] slot receives array of command parameters. 
+        @return bool true iff command detected.
+        """
+        ParametersFound['args'] = []
+        if CmdText == '' or CmdPattern == '':   # reject empty or disabled commands
+            return False;
+        head, sep, tail = CmdPattern.partition('*')
+        if sep == '':
+            return CmdText == CmdPattern    # simple case, no '*' in CmdPattern
+        pat, patTail, ellipsisSeen = '', '', False;
+        while sep != '':
+            ellipsisSeen = ellipsisSeen or tail[0:1] == '*'
+            tail = tail.lstrip('*')
+            if pat == '':   # text before 1st parameter (optional or not) must always be present.
+                pat += re.escape(head) + '(.*?)'
+                if ellipsisSeen: pat += '(?:' 
+            else:   # text before omitted optional parameter must be omitted as well.
+                if ellipsisSeen: pat += '(?:' 
+                pat += re.escape(head) + '(.*?)'
+            if ellipsisSeen:
+                patTail = ')?' + patTail;
+            head, sep, tail = tail.partition('*')
+        pat = '^' + pat + patTail + re.escape(head) + '$';
+        args = re.findall(pat, CmdText)
+        if len(args) > 0 and type(args[0]).__name__ in ('tuple', 'list'):
+            args = args[0];
+        ParametersFound['args'] = args;
+        return len(args) > 0
+
 ##############################################################
 # IMSI interface
 
@@ -1168,6 +1207,26 @@ class smscon_script:
 # %s custom script (initialized by %s version %s) """ % (NAME, NAME, VERSION) + """
 # Script will start on COM_CUSTOM command and executes asynchronously.
 # Script output (stdout and stderr) can be retrieved by COM_CUSTOMLOG command.
+#
+# Use non-consecutive '*' characters in COM_CUSTOM command definition to define placeholders.
+# Placeholder accept arbitrary text which is passed to this script as positional parameter.
+# Consecutive '**' indicate that this and all following placeholder are optional.
+# Example: The COM_CUSTOM command definition "MYCMD *,**,*" will recognize all these incoming SMS:
+# "MYCMD myparam1,myparam2,myparam3" --> "myparam1" "myparam2" "myparam3"
+# "MYCMD t h i s,that,"              --> "t h i s" "that" ""    (text for 3rd placeholder given but empty)
+# "MYCMD t h i s,that"               --> "t h i s" "that" ""    (text for 3rd placeholder not given)
+# "MYCMD a, b, c"                    --> "a" " b" " c"          (note the blanks)
+# "MYCMD a,b,c,d,e,f"                --> "a" "b" "c,d,e,f"      (right most placeholder is greedy)
+# It will not recognize:
+# "MYCMD,a,b,c"        (comma rather than space behind MYCMD)
+# "MYCMD"              (missing space behind MYCMD)
+# Consider to use a case statement to dispatch to any number of your own scripts.
+
+echo Number of parameters passed to this script: $#. The parameter values, one per line, are:
+for p in "$@"
+do
+    echo "$p"
+done
 
 # Here are some example commands you might consider:
 echo Process list on $(date):
